#ifndef _ALEA_HPP_
#define _ALEA_HPP_

#include <random>


double unif01()
{
  static std::mt19937 generateur(666);
  std::uniform_real_distribution<double> X(0,1);
  return X(generateur);
  // On simule une loi uniforme sur ]0,1[.
}


/*
  Definition de la classe (abstraite) correspondant aux variables aleatoires reelles.
  Le calcul de l'esperance, la variance, la fonction de repartition d'une variable
  se calculeront toujours de la meme maniere, quelle que soit la maniere dont est
  simulee la variable aleatoire.
  On pourra donc ecrire des classes heritant de va_reelle pour simuler des variables
  aleatoires des differentes lois classiques, et calculer leur variance, esperance,
  fonction de repartition...
*/

class va_reelle
{
public:
  virtual double operator() (void) const = 0;
  /* La syntaxe "virtual (...) =0" signifie que la fonction est
	 virtuelle pure (elle est declareee mais pas definie), ce qui rend
	 impossible la declaration d'une variable de type va_reelle. En revanche
	 une classe heritant de va_reelle pourra contenir une surcharge de
	 l'operateur ().
  */
  
  // Une methode calculant l'esperance de la variable par methode de Monte Carlo (avec n simulations)
  double E (int n) const;

  // Une methode calculant la variance de la variable par methode de Monte Carlo (avec n simulations)
  double V (int n) const;

  // Une methode calculant la fonction de repartition de la variable par methode de Monte Carlo (avec n simulations)
  double fdr(double x, int n) const;
};


double va_reelle::E (int n) const
{
  double mc = 0;
  for (int i=0; i<n; i++)
	mc += (*this)() / n;
  return mc;
}


double va_reelle::V (int n) const
{
  double somme = 0, somme_carres = 0;
  double x;
  for (int i=0; i<n; i++)
	{
	  x = (*this)();
	  somme += x/n;
	  somme_carres += x*x/n;
	}
  
  return somme_carres - somme*somme;
}


double va_reelle::fdr (double x,int n) const
{
  double mc = 0;
  for (int i=0; i<n; i++)
	if ((*this)() <= x)
	  mc += 1. / n;
  return mc;
}




#endif /* _ALEA_HPP_ */
